//// DropView.cpp
//

#define DEBUG 0
#include <Debug.h>

#include <Application.h>
#include <Roster.h>
#include <Window.h>
#include <Box.h>
#include <TextControl.h>
#include <Button.h>
#include <CheckBox.h>
#include <Node.h>
#include <NodeInfo.h>
#include <Bitmap.h>
#include <String.h>
#include <Beep.h>
#include <fs_attr.h>


#include "ZKDefs.h"
#include "EditView.h"

EditView::EditView(entry_ref a_ref)
: BBox(BRect(5,5,330,135), "EditView", B_FOLLOW_ALL, B_WILL_DRAW | B_FRAME_EVENTS)
{
	m_ref	=	a_ref;

	float str_width	=	StringWidth("Working dir");

	m_command	=	new BTextControl(BRect(5,50,318,70), "Command", "Command", "", NULL, B_FOLLOW_LEFT_RIGHT);	
	m_dir		=	new BTextControl(BRect(5,75,318,95), "Working dir", "Working dir", "", NULL, B_FOLLOW_LEFT_RIGHT);

	m_dir->			SetDivider(str_width-5);
	m_command->		SetDivider(str_width-5);
	
	m_filetypes_button	=	new BButton(BRect(93,115,318,140), "filetypes", "Edit Supported Filetypes, Icon, etc.", 
														new BMessage(ZOO_OPEN_FILETYPES), B_FOLLOW_LEFT_RIGHT);
	
	m_filetypes_button->SetFlags(m_filetypes_button->Flags() | B_FULL_UPDATE_ON_RESIZE);
	m_filetypes_button->MakeDefault(true);
	
	in_terminal	=	new BCheckBox (BRect(5,105,100,130), "in_terminal", "In Terminal", new BMessage(ZOO_IN_TERMINAL));
	in_terminal->SetValue(true);

	keep_open	=	new BCheckBox (BRect(5,125,100,145), "keep_open", "Keep Open", new BMessage(ZOO_KEEP_TERMINAL));
	keep_open->SetValue(true);
	
	AddChild(m_command);
	AddChild(m_dir);
	AddChild(in_terminal);
	AddChild(keep_open);
	AddChild(m_filetypes_button);
}

void 
EditView::AllAttached	(void)
{
	in_terminal->			SetTarget(this);
	keep_open->				SetTarget(this);
	m_filetypes_button->	SetTarget(this);
	
	Init();
}

void 
EditView::Draw	(BRect updateRect)	
{
	SetHighColor(Parent()->ViewColor());
	FillRect(updateRect);

	SetHighColor(0,0,0);

	float str_width		=	StringWidth("Working dir");
	float str_height	=	StringWidth("W");
	float name_width	=	StringWidth(m_ref.name);

	
	MovePenTo((str_width + (Bounds().Width()-(str_width))/2 - (name_width/2) - 7),
				(46 - 15 + (str_height/2) - 1));

	DrawString(m_ref.name);

	SetDrawingMode(B_OP_OVER);
	
	if (m_bitmap != NULL)
		DrawBitmap(m_bitmap, BRect((str_width/2-15), 15, (str_width/2+16), 46));
	else
	{
		// Grey fill
		SetHighColor(Parent()->ViewColor());
		FillRect(BRect((str_width/2-15), 15, (str_width/2+16), 46));
	}

	BBox::Draw(updateRect);
}

void 
EditView::FrameResized	(float width, float height)	
{
	BBox::FrameResized(width, height);

	Draw(Bounds());
}

EditView::~EditView()	
{
	delete m_bitmap;
}

void 
EditView::MessageReceived(BMessage * a_message)
{
	switch(a_message->what)
	{
		case ZOO_IN_TERMINAL:		Save();				break;
		case ZOO_KEEP_TERMINAL:		Save();				break;

		case ZOO_OPEN_FILETYPES:	FileTypes();		break;
				
		default:
				BBox::MessageReceived(a_message);
			break;			
	}
}

void 
EditView::Save	(void)
{
	BNode		node		(& m_ref);
	off_t		bytes	=	0;	
	
	BString	command	=	m_command->Text();
	BString	dir		=	m_dir->Text();
	
	if (command.Length() != 0)
	{
		bytes = node.WriteAttr("zook:command", B_STRING_TYPE, 0, command.String(), command.Length()+1);
		if (! bytes == command.Length()+1)
			beep();
	}
	else
		node.RemoveAttr("zook:command");
	
	if (dir.Length() != 0)
	{
		bytes = node.WriteAttr("zook:dir", B_STRING_TYPE, 0, dir.String(), dir.Length()+1);
		if (! bytes == dir.Length()+1)
			beep();
	}
	else
		node.RemoveAttr("zook:dir");
	
	if (in_terminal->Value())
		bytes = node.WriteAttr("zook:terminal", B_STRING_TYPE, 0, "Y", 2);
	else
		node.RemoveAttr("zook:terminal");

	if (keep_open->Value())
		bytes = node.WriteAttr("zook:keepopen", B_STRING_TYPE, 0, "Y", 2);
	else
		node.RemoveAttr("zook:keepopen");

}

void 
EditView::Init	(void)
{
	BNode		node		(& m_ref);
	BNodeInfo	nodeinfo	(& node);
	
	m_bitmap	=	new BBitmap	(BRect(0,0,31,31), B_COLOR_8_BIT, true);
		
	if (BNodeInfo::GetTrackerIcon(& m_ref, m_bitmap, B_LARGE_ICON) != B_OK)
		m_bitmap = NULL;

	char * buffer	=	new char [4096];
	off_t	bytes	=	0;
	
	bytes = node.ReadAttr("zook:command", B_STRING_TYPE, 0, buffer, 4095);
	if (bytes >= 0)
	{
		buffer[bytes] = '\0';
		m_command->SetText(buffer);
	}
	
	bytes = node.ReadAttr("zook:dir", B_STRING_TYPE, 0, buffer, 4095);
	if (bytes >= 0)
	{
		buffer[bytes] = '\0';
		m_dir->SetText(buffer);
	}
	
	attr_info	info;
	if (node.GetAttrInfo("zook:terminal", & info) == B_OK)
		in_terminal->SetValue(true);
	else
		in_terminal->SetValue(false);
	
	
	if (node.GetAttrInfo("zook:keepopen", & info) == B_OK)
		keep_open->SetValue(true);
	else
		keep_open->SetValue(false);
	
	delete [] buffer;
}

void 
EditView::FileTypes	(void)
{
	BMessage	msg	(B_REFS_RECEIVED);
	msg.AddRef("refs", & m_ref);
	be_roster->Launch("application/x-vnd.Be-MIMA", & msg);
}
