// ZKWindow.cpp

#define DEBUG 0
#include <Debug.h>

#include <stdio.h>
#include <stdlib.h>

#include <Application.h>
#include <Roster.h>
#include <InterfaceKit.h>
#include <String.h>
#include <FindDirectory.h>
#include <Directory.h>
#include <Path.h>
#include <File.h>
#include <Beep.h>
#include <Deskbar.h>

#include "ZKDefs.h"
#include "ZKWindow.h"
#include "EditView.h"
#include "IconMenu.h"

ZKWindow::ZKWindow(BRect a_winframe)
: BWindow(BRect(300,200,636,360), "ZooKeeper", B_TITLED_WINDOW, B_NOT_V_RESIZABLE )
{
	PRINT(("ZKWindow()\n"));
	
	m_editbox	=	NULL;
	topbox		=	NULL;
	
//	UI
	m_menubar		= new BMenuBar(BRect(0,0,195,20), "menubar");
	m_appmenu		= new BMenu("App");
	
	m_iconmenu		= new IconMenu(m_appmenu);
	m_menubar->		AddItem(m_iconmenu);
	
	m_aboutitem 	= new BMenuItem("About" B_UTF8_ELLIPSIS, new BMessage(B_ABOUT_REQUESTED));
	m_help_item		= new BMenuItem("Help" B_UTF8_ELLIPSIS, new BMessage(ZOO_HELP_REQUESTED));
	m_quititem 		= new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q');

	m_appmenu->		AddItem(m_aboutitem);
	m_appmenu->		AddItem(m_help_item);
	m_appmenu->		AddSeparatorItem();
	m_appmenu->		AddItem(m_quititem);

	float width, height;
	m_menubar->	GetPreferredSize(& width, & height);
	ResizeBy(0,height+2);
	
	topbox	=	new BBox(BRect(1,height+2,Bounds().right,Bounds().bottom), "view", B_FOLLOW_ALL_SIDES, B_NAVIGABLE);

//	m_dropbox	=	new	BBox(BRect(5,5,330,155), "Dropbox", B_FOLLOW_ALL);
//	m_dropbox->	SetLabel("Drop App or Script here");
	
//	topbox->	AddChild(m_dropbox);	// .. with this
	
	app_info	my_app_info;
	be_app->GetAppInfo(& my_app_info);
	
	m_editbox	=	new	EditView(my_app_info.ref);

	topbox->	AddChild(m_editbox);

	m_editbox->MoveTo(5,5);
	m_editbox->ResizeTo(topbox->Bounds().Width()-10, topbox->Bounds().Height()-10);
	//

	AddChild(m_menubar);
	AddChild(topbox);

	MakeSettingsFolder();
	ReadSettings();
	
	SetSizeLimits(336, 800, 100, 15000);
}

ZKWindow::~ZKWindow()	
{
	
}

void 
ZKWindow::MessageReceived		(BMessage * a_message)
{
	PRINT(("ZKWindow::MessageReceived()\n"));
	
	switch(a_message->what)
	{
		case ZOO_HELP_REQUESTED:			be_app->PostMessage(a_message);			break; 
		case B_ABOUT_REQUESTED:				be_app->PostMessage(a_message);			break;

		default:	BWindow::MessageReceived(a_message);	break;			
	}
}

bool
ZKWindow::QuitRequested	(void)
{
	SaveSettings();
	m_editbox->Save();

	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

status_t 
ZKWindow::MakeSettingsFolder	(void)
{
	status_t	status;
	BPath		path;
	if ((status = find_directory(B_USER_SETTINGS_DIRECTORY, & path)) != B_OK)
		return status;
	
	BEntry 		entry	(path.Path());

	// settings
	if (entry.Exists()	==	false	||	entry.IsDirectory()	==	false)	
		return B_ERROR;
	
	BDirectory	mother	(path.Path());
	BDirectory	baby;
	
	// Kirilla
	path.SetTo(path.Path(), "Kirilla");
	entry.SetTo(path.Path());
	if (! entry.Exists())
	{
		status	=	mother.CreateDirectory("Kirilla", & baby);
		if (status != B_OK && status != B_FILE_EXISTS)				return status;
	}
	else 
		if (! entry.IsDirectory())
			return B_FILE_EXISTS;
	
	if ((status = mother.SetTo(path.Path())) != B_OK)			return status;

	// Springboard
	path.SetTo(path.Path(), "ZooKeeper");
	entry.SetTo(path.Path());
	if (! entry.Exists())
	{
		status	=	mother.CreateDirectory("ZooKeeper", & baby);
		if (status != B_OK && status != B_FILE_EXISTS)				return status;
	}
	else 
		if (! entry.IsDirectory())
			return B_FILE_EXISTS;
	
	if ((status = mother.SetTo(path.Path())) != B_OK)			return status;

	entry.SetTo(path.Path());
	if (entry.Exists()	&&	entry.IsDirectory())	return B_OK;
	else											return B_ERROR;
}

status_t
ZKWindow::ReadSettings	(void)
{
	status_t	status;
	BMessage	settings;
	BPath		path;
	BFile		file;
	BRect 		frame;

	// get path to settings-file
	if ((status = find_directory(B_USER_SETTINGS_DIRECTORY, & path)) != B_OK)
		return status;

	if ((status = path.SetTo(path.Path(),"Kirilla/ZooKeeper/settings")) != B_OK)
		return status;
		
	if ((status	= file.SetTo(path.Path(), B_READ_ONLY)) != B_OK)
		return status;

	// get message from file
	if ((status	=	settings.Unflatten(& file)) != B_OK)
		return status;

	// get settings from message
	if (settings.FindRect("winframe", & frame) == B_OK)
	{
		MoveTo(frame.LeftTop());
		ResizeTo(frame.Width(), frame.Height());
	}

	return B_OK;
}

status_t 
ZKWindow::SaveSettings	(void)
{
	status_t	status;
	BMessage 	settings;
	BPath		path;
	BFile		file;

	// add settings to message
	settings.AddRect("winframe", Frame());
	
	// get path to settings-file
	if ((status = find_directory(B_USER_SETTINGS_DIRECTORY, & path)) != B_OK)
		return status;

	if ((status = path.SetTo(path.Path(),"Kirilla/ZooKeeper/settings")) != B_OK)
		return status;
		
	if ((status	= file.SetTo(path.Path(), B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE)) != B_OK)
		return status;

	if ((status	= settings.Flatten(& file))	!=	B_OK)
		return status;

	return B_OK;
}

